using System;
using System.Collections;
using org.bouncycastle.asn1;
using org.bouncycastle.asn1.x509;

namespace org.bouncycastle.asn1.pkcs
{
	/**
	 * a PKCS#7 signer info object.
	 */
	public class SignerInfo: DEREncodable
	{
		private DERInteger              version;
		private IssuerAndSerialNumber   issuerAndSerialNumber;
		private AlgorithmIdentifier     digAlgorithm;
		private ASN1Set                 authenticatedAttributes;
		private AlgorithmIdentifier     digEncryptionAlgorithm;
		private ASN1OctetString         encryptedDigest;
		private ASN1Set                 unauthenticatedAttributes;

		public static SignerInfo getInstance(
			Object  o)
		{
			
			if (typeof(SignerInfo).IsInstanceOfType(o))
			{
				return (SignerInfo)o;
			}
			else if (typeof(ASN1Sequence).IsInstanceOfType(o))
			{
				return new SignerInfo((ASN1Sequence)o);
			}

			throw new ArgumentException("unknown object in factory");
		}

		public SignerInfo(
			DERInteger              version,
			IssuerAndSerialNumber   issuerAndSerialNumber,
			AlgorithmIdentifier     digAlgorithm,
			ASN1Set                 authenticatedAttributes,
			AlgorithmIdentifier     digEncryptionAlgorithm,
			ASN1OctetString         encryptedDigest,
			ASN1Set                 unauthenticatedAttributes)
		{
			this.version = version;
			this.issuerAndSerialNumber = issuerAndSerialNumber;
			this.digAlgorithm = digAlgorithm;
			this.authenticatedAttributes = authenticatedAttributes;
			this.digEncryptionAlgorithm = digEncryptionAlgorithm;
			this.encryptedDigest = encryptedDigest;
			this.unauthenticatedAttributes = unauthenticatedAttributes;
		}

		public SignerInfo(
			ASN1Sequence seq)
		{
			IEnumerator     e = seq.getObjects();
			e.MoveNext();
			version = (DERInteger)e.Current;
			e.MoveNext();
			issuerAndSerialNumber = IssuerAndSerialNumber.getInstance(e.Current);
			e.MoveNext();
			digAlgorithm = AlgorithmIdentifier.getInstance(e.Current);

			Object obj = e.Current;

			if (typeof(ASN1TaggedObject).IsInstanceOfType(obj))
			{
				authenticatedAttributes = ASN1Set.getInstance((ASN1TaggedObject)obj, false);

				digEncryptionAlgorithm = AlgorithmIdentifier.getInstance(e.Current);
			}
			else
			{
				authenticatedAttributes = null;
				digEncryptionAlgorithm = AlgorithmIdentifier.getInstance(obj);
			}

			encryptedDigest = DEROctetString.getInstance(e.Current);

			if (e.MoveNext())
			{
				unauthenticatedAttributes = ASN1Set.getInstance((ASN1TaggedObject)e.Current, false);
			}
			else
			{
				unauthenticatedAttributes = null;
			}
		}

		public DERInteger getVersion()
		{
			return version;
		}

		public IssuerAndSerialNumber getIssuerAndSerialNumber()
		{
			return issuerAndSerialNumber;
		}

		public ASN1Set getAuthenticatedAttributes()
		{
			return authenticatedAttributes;
		}

		public AlgorithmIdentifier getDigestAlgorithm()
		{
			return digAlgorithm;
		}

		public ASN1OctetString getEncryptedDigest()
		{
			return encryptedDigest;
		}

		public AlgorithmIdentifier getDigestEncryptionAlgorithm()
		{
			return digEncryptionAlgorithm;
		}

		public ASN1Set getUnauthenticatedAttributes()
		{
			return unauthenticatedAttributes;
		}

		/**
		 * <pre>
		 *  SignerInfo ::= SEQUENCE {
		 *      version Version,
		 *      issuerAndSerialNumber IssuerAndSerialNumber,
		 *      digestAlgorithm DigestAlgorithmIdentifier,
		 *      authenticatedAttributes [0] IMPLICIT Attributes OPTIONAL,
		 *      digestEncryptionAlgorithm DigestEncryptionAlgorithmIdentifier,
		 *      encryptedDigest EncryptedDigest,
		 *      unauthenticatedAttributes [1] IMPLICIT Attributes OPTIONAL
		 *  }
		 *
		 *  EncryptedDigest ::= OCTET STRING
		 *
		 *  DigestAlgorithmIdentifier ::= AlgorithmIdentifier
		 *
		 *  DigestEncryptionAlgorithmIdentifier ::= AlgorithmIdentifier
		 * </pre>
		 */
		public DERObject getDERObject()
		{
			ASN1EncodableVector v = new ASN1EncodableVector();

			v.add(version);
			v.add(issuerAndSerialNumber);
			v.add(digAlgorithm);

			if (authenticatedAttributes != null)
			{
				v.add(new DERTaggedObject(false, 0, authenticatedAttributes));
			}

			v.add(digEncryptionAlgorithm);
			v.add(encryptedDigest);

			if (unauthenticatedAttributes != null)
			{
				v.add(new DERTaggedObject(false, 1, unauthenticatedAttributes));
			}

			return new DERSequence(v);
		}
	}

}