using System;
using System.IO;
using System.Collections;
using org.bouncycastle.io;
using org.bouncycastle.asn1;

namespace org.bouncycastle.asn1.x509
{
	/**
	* The object that contains the public key stored in a certficate.
	* <p>
	* The getEncoded() method in the public keys in the JCE produces a DER
	* encoded one of these.
	*/
	public class SubjectPublicKeyInfo: DEREncodable
	{
		private AlgorithmIdentifier     algId;
		private DERBitString            keyData;

		public static SubjectPublicKeyInfo getInstance(
			ASN1TaggedObject obj,
			bool          flagExplicit)
		{
			return getInstance(ASN1Sequence.getInstance(obj, flagExplicit));
		}

		public static SubjectPublicKeyInfo getInstance(
			Object  obj)
		{
			
			if (typeof(SubjectPublicKeyInfo).IsInstanceOfType(obj))
			{
				return (SubjectPublicKeyInfo)obj;
			}
			else if (typeof(ASN1Sequence).IsInstanceOfType(obj))
			{
				return new SubjectPublicKeyInfo((ASN1Sequence)obj);
			}

			throw new ArgumentException("unknown object in factory");
		}

		public SubjectPublicKeyInfo(
			AlgorithmIdentifier algId,
			DEREncodable        publicKey)
		{
			this.keyData = new DERBitString(publicKey);
			this.algId = algId;
		}

		public SubjectPublicKeyInfo(
			AlgorithmIdentifier algId,
			byte[]              publicKey)
		{
			this.keyData = new DERBitString(publicKey);
			this.algId = algId;
		}

		public SubjectPublicKeyInfo(
			ASN1Sequence  seq)
		{
			IEnumerator         e = seq.getObjects();
			e.MoveNext();
			this.algId = AlgorithmIdentifier.getInstance(e.Current);
			e.MoveNext();
			this.keyData = (DERBitString)e.Current;
		}

		public AlgorithmIdentifier getAlgorithmId()
		{
			return algId;
		}

		/**
		* for when the public key is an encoded object - if the bitstring
		* can't be decoded this routine throws an IOException.
		*
		* @exception IOException - if the bit string doesn't represent a DER
		* encoded object.
		*/
		public DERObject getPublicKey()
			//throws IOException
		{
			MemoryStream    bIn = new MemoryStream(keyData.getBytes());
			DERInputStream          dIn = new DERInputStream(bIn);

			return dIn.readObject();
		}

		/**
		* for when the public key is raw bits...
		*/
		public DERBitString getPublicKeyData()
		{
			return keyData;
		}

		/**
		* <pre>
		* SubjectPublicKeyInfo ::= SEQUENCE {
		*                          algorithm AlgorithmIdentifier,
		*                          publicKey BIT STRING }
		* </pre>
		*/
		public DERObject getDERObject()
		{
			ASN1EncodableVector  v = new ASN1EncodableVector();

			v.add(algId);
			v.add(keyData);

			return new DERSequence(v);
		}
	}

}